
-- AD7818 A/D Converter controller module
-- To be used with Simplex G-LINK S-LINK LDC  
-- Created on 09. June. 1999
-- Developer: Zoltan Meggyesi

-- LD (Data) bits in test mode (UTDO_N low)
-- 31,                30               29..10, 9..0
-- Light/Temperature, Over Temperature reserved Laser power/Temperature
-- Conversion of D[31..26] to dbm: decimal value-> X
-- Pwr = 10*log(X*2*2.44/3100)    
-- If R21 is removed (voltage div/2) then 10*log(X*2.44/3100) 
--====================================================================================
-- Modifications: 
-- 09.06.99: First Version
-- 14.06.99: Data output: d[31..26]-> Reserved D[25..16]->pwr 
--           D[15..8]->temperature (celsius) D[7..6] Temperature fraction 0.25..0.75
--           D5-> Over temperature, above 50cels. D[4..0]->01100
-- 16.06.99: Power is displayed in dBm using ROM look-up table.
--           Data output: d[31..24]-> Reserved D[23..16]-> -10*pwr in dBm 
--           D[15..8]->temperature (celsius) D[7..6] Temperature fraction 0.25..0.75
--           D5-> Over temperature, above 50cels. D[4..0]->01100

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     IEEE.STD_LOGIC_ARITH.ALL;
USE     IEEE.STD_LOGIC_UNSIGNED.ALL;

entity sensor is

  port(
        CLK          : in    std_logic;  -- Clock signal, 40Mhz
        TEST         : in    std_logic;  -- Sensor Mode
        
        DIO          : inout std_logic;  -- Sensor Data Line 
        OTI_N        : in    std_logic;  -- Over Temperature 
        CONVST_N     : out   std_logic;  -- Conversion Start;
        SCLK         : out   std_logic;  -- Sensor Clock - 5Mhz
        RDWR         : out   std_logic;  -- Read/Write Sensor   

        D_OUT        : out   std_logic_vector(31 downto 0); -- Data out 
        WEN_N        : out   std_logic);  -- Write enable

end sensor;

ARCHITECTURE behavioural OF sensor IS

COMPONENT lpm_counter
   GENERIC (LPM_WIDTH:POSITIVE);
      PORT (
      clock: IN STD_LOGIC;
      sset:  IN STD_LOGIC;
      sclr:  IN STD_LOGIC;
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0) );
END COMPONENT;

COMPONENT lpm_shiftreg
   GENERIC (LPM_WIDTH: POSITIVE);
   PORT (
      data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0);
      clock: IN STD_LOGIC;
      shiftin: IN STD_LOGIC;
      load: IN STD_LOGIC;
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0) );
END COMPONENT;

COMPONENT lpm_rom
   GENERIC (
      LPM_WIDTH: POSITIVE;
      LPM_WIDTHAD: POSITIVE;
      LPM_FILE: STRING);
   PORT (
      address: IN STD_LOGIC_VECTOR(LPM_WIDTHAD-1 DOWNTO 0);
      inclock: IN STD_LOGIC := '1';
      outclock: IN STD_LOGIC := '1';
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;

TYPE m_state is (STDBY, WRCH1, CONVCH1, RDCH1, WRTS, CONVTS, RDTS, STOP); 

signal curr_m, next_m: m_state; -- Main state machine register
signal stopread, stopwrite, smset, smclr, smclk: std_logic;
signal rwq, wc : std_logic_vector(2 downto 0);  
signal smq: std_logic_vector(9 downto 0);
signal rwclk, wshin, rshin, wshload, rshload: std_logic; -- Write shifterg 
signal wshd, wshq, temperature, power, pwrlatch: std_logic_vector(7 downto 0);
signal rshd, rshq, outreg : std_logic_vector(9 downto 0);
signal rcena,  rcset, rcclr : std_logic; -- Read counter signals
signal wcena, wcset, wcclr: std_logic;    -- Write counter signals
signal otr: std_logic;    -- Over temperature latch
signal rc: std_logic_vector(3 downto 0);

begin

-- Read Clock: 5 mhz  -----------------------------------------------

RWCLOCK: lpm_counter -- Clock that goes to converter. 
           generic map (LPM_WIDTH => 3)
           port map    (CLK, stopread, stopwrite, rwq); 

stopread<='0'; stopwrite<='0';
rwclk <= not(rwq(2));

-- State Machine Clock: 1064*25 ns ~25us ----------------------------

SMCLOCK: lpm_counter 
           generic map (LPM_WIDTH => 10)
           port map    (CLK, smset, smclr, smq); 

smclk <= smq(9);
smset <='0';
smclr <='0';

-- Main State Machine; Clked by 25us time period --------------------

m_reg: process (smclk) begin
  if (smclk'event and smclk='1') then
    curr_m <= next_m;
  end if;
end process m_reg;

m_machine: process (curr_m) begin
  case curr_m is                   -- Reads only once after switched 
                                   -- To test mode
    when STDBY =>                  -- Not in test mode
      if (TEST = '0') then next_m <= WRCH1;
      else next_m <= STDBY;
      end if;
   
    when WRCH1 =>                  -- Write Channel 1
      if (TEST = '1') then next_m <= STDBY;
      else next_m <= CONVCH1;
      end if;

    when CONVCH1 => next_m <= RDCH1;    -- Wait for conversion

    when RDCH1 =>                       -- Read Channel 1
      next_m <= WRTS;

    when WRTS =>                   -- Write temperature sensor
      if (TEST = '1') then next_m <= STDBY;
      else next_m <= CONVTS;
      end if;

    when CONVTS => next_m <= RDTS;      -- Wait for temp. conversion

    when RDTS =>                        -- Read temperature sensor
      next_m <= STOP;
    
    when STOP =>                        -- Wait for end of Test mode
      if (TEST = '1') then next_m <= STDBY;
      else next_m <= STOP;
      end if;

  end case;
end process m_machine;

-- Main State Machne output: Read/Write counter enable ------------------

m_out: process (rwclk) begin
  if (rwclk'event and rwclk='1') then
    case curr_m is
      when STDBY  => rcena   <= '0'; -- Disable read & write
                     wcena   <= '0'; 
                     wshd    <= ('0','0','0','0','0','0','0','1'); 
      when WRCH1  => wcena   <= '1'; -- Enable Write operation
                     rcena   <= '0';
                     wshd    <= (others => '0');
      when RDCH1  => rcena   <= '1'; -- Enable read operation
                     wcena   <= '0';
                     wshd    <= ('0','0','0','0','0','0','0','0'); 
      when WRTS   => wcena   <= '1';
                     rcena   <= '0';
                     wshd    <= (others => '0');
      when RDTS   => rcena   <= '1';
                     wcena   <= '0';
                     wshd    <= (others => '0');
      when others => rcena   <= '0'; -- Disable read & write
                     wcena   <= '0';
                     wshd    <= (others => '0');
  end case;
end if;
end process m_out;

unreg_out: process (curr_m, rwq(2), wcset, wshq(7)) begin
  case curr_m is
    when WRCH1 => SCLK <=not(rwq(2) and (not(wcset)));
                  RDWR <= '0';
                  DIO  <= wshq(7);   -- Leftmost bit of Write shifteg      
                  wshload <= '0';        
    when WRTS  => SCLK <=not(rwq(2) and (not(wcset)));
                  RDWR <= '0';
                  DIO  <= wshq(7);
                  wshload <= '0';
    when RDCH1 => SCLK <=not(rwq(2) and (not(rcset)));
                  RDWR <= not rcset;
                  DIO  <= 'Z';
                  wshload <= '1';
    when RDTS  => SCLK <=not(rwq(2) and (not(rcset)));
                  RDWR <= not rcset;
                  DIO  <= 'Z';
                  wshload <= '1';
    when others=> SCLK <= '1';
                  RDWR <= '0';
                  DIO  <= 'Z';
                  wshload <= '1';

  end case;
end process unreg_out;

-- Write counter and output enable -------------------------------------

WCNT: lpm_counter 
           generic map (LPM_WIDTH => 3)
           port map    (rwclk, wcset, wcclr, wc);       

wcclr <= not(wcena);  -- Clear when not enabled (outside write operation)

write_cnt_set: process (rwclk) begin
  if (rwclk'event and rwclk='1') then
    wcset <= (wc(2) and wc(1));       -- Stay "111" when full
  else wcset <= wcset;
  end if;
end process write_cnt_set; 

-- Write Shift Register ------------------------------------------------

WSHIFT: lpm_shiftreg
             generic map (LPM_WIDTH => 8)
             port map (wshd, rwclk, wshin, wshload, wshq); 

   -- Shift data out when counter is enabled and not yet full
wshin   <= '0'; -- Not used
 
-- CONVST_N ------------------------------------------------------------
-- "Mode 2" power saving mode, high before conversion
-- High at the  middle of the write states in order to start conversion
     
conversion: process (CLK) begin    
  if (CLK'event and CLK='1') then
    case curr_m is
      when WRCH1 => CONVST_N <= not (smq(9) or smq(8) or smq(7));
      when WRTS  => CONVST_N <= not (smq(9) or smq(8) or smq(7));
      when others =>CONVST_N <= '0';                
    end case;               
  end if;
end process conversion;              


-- Read Counter --------------------------------------------------------
  
RCNT: lpm_counter 
           generic map (LPM_WIDTH => 4)  
           port map    (rwclk, rcset, rcclr, rc); 

rcclr <= not(rcena);  -- clear when not enabled (outside write operation)

read_cnt_set: process (rwclk) begin    -- Go to F when 10 words are read
  if (rwclk'event and rwclk='1') then
    case rc is
      when "1000" => rcset <='1'; -- When 8
      when "1001" => rcset <='1'; -- When 9;
      when "1111" => rcset <='1'; -- Rcset stays high
      when others => rcset <='0';
    end case;
  end if;
end process read_cnt_set; 

-- Read Shift Register ------------------------------------------------

RSHIFT: lpm_shiftreg
             generic map (LPM_WIDTH => 10)
             port map (rshd, rwclk, rshin, rshload, rshq); 

rshd <= (others => '0');    -- Load Not used;
rshload <= '0'; -- Not used;

rshin <= DIO;   -- Shift in data from DIO;

-- Over Temperature indicator -----------------------------------------
-- OTI_N is reset at the start of the read operation (WHY ?!?!?!), so
-- it has to be latched so as to send its value together with outreg
-- after the read cycles. 

alarm :process (rwclk) begin  
  if (rwclk'event and rwclk='1') then
    if (curr_m = RDCH1) then otr <= '0'; -- Clear Over-Temp. reg.
    elsif (OTI_N = '0') then otr <= '1'; -- Set if OTI_N goes low
    else otr <= otr;                     -- Keep value until clear
    end if;
  end if;
end process alarm;

-- Temperature and Power -----------------------------------------------

temperature <= outreg(9 downto 2) - 103;    -- Temperature Conversion

CONVERTER: lpm_rom 
             generic map (LPM_WIDTH=>8, LPM_WIDTHAD=>10, LPM_FILE=>"power.mif")
             port map (outreg, rwclk, rwclk, power);


-- Parallel Data Output -----------------------------------------------

shiftreg_read: process (rwclk) begin    
  if (rwclk'event and rwclk='1') then
    case rc is
      when "1001" => outreg <= rshq; -- When finished serial read
      when others => outreg <= outreg;
    end case;
  end if;
end process shiftreg_read;

data_out: process (rwclk) begin      -- Write CH1 to pwrlatch and keep value   
  if (rwclk'event and rwclk='1') then
    case curr_m is
      when RDCH1 =>   -- Write in Read Channel 1 state
        pwrlatch <= power; -- Power
      when others  =>
        pwrlatch <= pwrlatch; -- Keep value (power)
    end case;
  end if;
end process data_out;

D_OUT(31 downto 24)  <= ('0', '0', '0', '0', '0', '0', '0', '0'); 
D_OUT(23 downto 16)  <= pwrlatch;           -- power
D_OUT(15 downto 8)   <= temperature;
D_OUT( 7 downto 6)   <= outreg(1 downto 0); -- 1/2, 1/4 degrees
D_OUT(5)             <= otr;    -- over temperature
D_OUT(4 downto 0)    <= ('0', '1', '1', '0', '0');

-- WEN_N for S-LINK goes low for CLK time at the end of "RDTS" State ---

write: process (CLK) begin    -- Go to F when 10 words are read
  if (CLK'event and CLK='1') then
     case curr_m is
       when RDTS =>
         WEN_N <= smq(9) or smq(8) or smq(7) or smq(6) or smq(5) or
                  smq(4) or smq(3) or smq(2) or smq(1) or smq(0);
                  -- Low when state machine clock counter is zero.           
       when others => WEN_N <='1';
     end case;
  end if;
end process write;

end behavioural;

